/**
 * Javier Abelln, 15 Mayo 2006
 */
package chuidiang.paneles;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;

/**
 * Panel con imagen que se puede arrastrar.
 * 
 * @author Chuidiang
 */
public class PanelImagenPanoramica extends JComponent
{
    /** Indica si se limita el movimiento para que no dejar espacios sin foto
     * en el panel.
     */
    boolean limitarMovimiento = true;

    /**
     * serial uid
     */
    private static final long serialVersionUID = 3904958634051319353L;

    /** Crea una intanciad esta clase. La imagen es la que usara para mostrar */
    public PanelImagenPanoramica(Image imagen)
    {
        setImagen(imagen);
        suscribeteARaton();
    }

    /** Crea una instancia de esta clase, sin imagen para mostrar */
    public PanelImagenPanoramica()
    {
        suscribeteARaton();
    }

    /** Se suscribe a los movimientos de ratn para realizar el arrastrado
     * de la imagen
     */
    private void suscribeteARaton()
    {
        this.addMouseMotionListener(new MouseMotionListener()
        {
            /** Marca si se est empezando a arrastrar */
            private boolean arrastrando = false;

            /** Ultima x del ratn */
            private int x;

            /** Ultima y del raton */
            private int y;

            /** Si el ratn se mueve sin arrastrar, marca que no estamos
             * arrastrando.
             */
            public void mouseMoved(MouseEvent e)
            {
                arrastrando = false;
            }

            /** Si se esta arranstrando, se calcula la nueva posicionx y 
             * posiciony para la imagen
             */
            public void mouseDragged(MouseEvent e)
            {
                // Si no se estaba arranstrando antes, slo guardamos la x,y
                // del ratn.
                if (arrastrando == false)
                {
                    x = e.getX();
                    y = e.getY();
                    arrastrando = true;
                } else
                {
                    // Se calcula la nueva posicion en funcin de lo que se 
                    // ha movido el ratn desde la ltima vez.
                    posicionX = posicionX + (e.getX() - x);
                    posicionY = posicionY + (e.getY() - y);
                    x = e.getX();
                    y = e.getY();
                    revisarNuevaPosicion();
                    repaint();
                }
            }

        });
    }

    /** Verifica si el arrastre de la imagen hace que queden zonas del componente
     * sin imagen. Si el flag limitarMovimiento dice que esto no est permitido,
     * corrige la posicin.
     */
    private void revisarNuevaPosicion()
    {
        if (!isLimitarMovimiento())
            return;

        if (posicionX > 0)
            posicionX = 0;
        if (posicionY > 0)
            posicionY = 0;

        if (posicionX + imagen.getWidth(this) < getWidth())
            posicionX = getWidth() - imagen.getWidth(this);
        if (posicionY + imagen.getHeight(this) < getHeight())
            posicionY = getHeight() - imagen.getHeight(this);
    }

    /** Imagen a dibujar */
    private Image imagen = null;

    /** Posicion x de la imagen */
    private int posicionX = 0;

    /** Posicion y de la imagen */
    private int posicionY = 0;

    /** Repintado del componente. Pinta la imagen y luego llama a super.paint().
     * 
     */
    public void paint(Graphics g)
    {
        if (imagen != null)
        {
            System.out.println(posicionX);
            g.drawImage(imagen, posicionX, posicionY, imagen.getWidth(this),
                    imagen.getHeight(this), this);
        }
        super.paint(g);
    }

    /** Fija la imagen que hay que dibujar. */
    public void setImagen(Image imagen)
    {
        this.imagen = imagen;
    }

    /** Devuelve si se est limitando el movimiento */
    public boolean isLimitarMovimiento()
    {
        return limitarMovimiento;
    }

    /** Fija si se debe limitar el movimiento. */
    public void setLimitarMovimiento(boolean limitarMovimiento)
    {
        this.limitarMovimiento = limitarMovimiento;
    }

}
